import 'dart:math';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:sport/bean/comment.dart';
import 'package:sport/bean/post.dart';
import 'package:sport/bean/post_user.dart';
import 'package:sport/pages/social/gallery_photo_view.dart';
import 'package:sport/pages/social/notification.dart';
import 'package:sport/pages/social/post_comment.dart';
import 'package:sport/provider/lib/provider_widget.dart';
import 'package:sport/provider/lib/view_state_lifecycle.dart';
import 'package:sport/provider/post_detail_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/utils/DateFormat.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/button_primary.dart';
import 'package:sport/widgets/decoration.dart';
import 'package:sport/widgets/dialog/modal_bottom_action.dart';
import 'package:sport/widgets/error.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';
import 'package:sport/widgets/text_input.dart' as input;

class PostDetailPage extends StatefulWidget {
  final List posts;
  final Post post;
  final bool comment;

  PostDetailPage(this.post, this.comment, this.posts);

  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends ViewStateLifecycle<PostDetailPage, PostDetailModel> {
  FocusNode _focusNode;
  ScrollController _controller;
  int _inputType = 0;
  Post _post;

  static const List<String> _sort = ["最新评论", "点赞最多"];
  String _sortType = _sort[0];

  final GlobalKey _key = GlobalKey();
  double _offset = 0;

  var _padding = Space(
    width: 5,
  );

  var _scrollListener;

  Comment _toComment;
  PostUser _toUser;

  @override
  PostDetailModel createModel() => PostDetailModel(widget.post);

  @override
  void initState() {
    super.initState();
    _post = widget.post;

    _focusNode = FocusNode();

    _scrollListener = () {
      resetInput();
    };
    _controller = ScrollController()..addListener(_scrollListener);

    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      RenderBox box = _key.currentContext.findRenderObject();
      var offset = box.localToGlobal(Offset.zero);
      final double statusBarHeight = MediaQuery.of(context).padding.top;
      _offset = offset.dy - kToolbarHeight - statusBarHeight;
    });

    if (widget.comment) {
      setState(() {
        _inputType = 1;
      });
      WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
        FocusScope.of(context).requestFocus(_focusNode);
      });
    }
  }

  void _scrollToComment() {
    _controller.animateTo(_offset, duration: Duration(milliseconds: 500), curve: Curves.ease);
  }

  void resetInput() {
    if (_focusNode.hasFocus) {
      _focusNode.unfocus();
      setState(() {
        _toComment = null;
        _toUser = null;
        _inputType = 0;
      });
    }
  }

  void readyInput() {
    setState(() {
      _inputType = 1;
    });
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      FocusScope.of(context).requestFocus(_focusNode);
    });
  }

  @override
  void dispose() {
    _controller?.dispose();
    _focusNode?.dispose();
    super.dispose();
  }

  // /**
  //  * 1: 评论
  //  * 2: 点赞
  //  */
  _updatePost(int type, int num) {
    if (widget.posts == null) {
      _post?.toggleLike();
      return;
    }
    widget.posts.forEach((element) {
      if (element.id == widget.post.id) {
        if (type == 1) {
          element.commentCount = element.commentCount + num;
        } else {
          element.toggleLike();
        }
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        title: Text(
          "帖子详情",
          style: titleStyle,
        ),
        leading: buildBackButton(context),
        actions: <Widget>[],
      ),
      body: ProviderWidget<PostDetailModel>(
          model: model,
          onModelReady: (model) => model.initData(),
          builder: (_, model, __) {
            return NotificationListener(
                onNotification: (notification) {
                  if (notification is CommentNotification) {
                    if (notification.type == CommentNotification.TYPE_DEL) {
                      model.list.removeWhere((element) => element.id == notification.comment.id);
                      setState(() {});
                    }
                    _updatePost(1, notification.type);
                    return true;
                  }
                  if (notification is CommentInputNotification) {
                    _toComment = notification.comment;
                    _toUser = notification.comment.socialInfo;
                    readyInput();
                    return true;
                  }
                  return false;
                },
                child: Column(
                  children: <Widget>[
                    Expanded(
                      flex: 1,
                      child: EasyRefresh.custom(
                        controller: model.refreshController,
                        enableControlFinishRefresh: true,
                        enableControlFinishLoad: true,
                        scrollController: _controller,
                        onRefresh: () => model.refresh(),
                        onLoad: model.isIdle ? () => model.loadMore() : null,
                        header: buildClassicalHeader(),
                        footer: buildClassicalFooter(),
                        slivers: <Widget>[
                          SliverToBoxAdapter(
                            child: _buildPostDetailWidget(widget.post),
                          ),
                          if (model.isBusy)
                            SliverToBoxAdapter(
                              child: RequestLoadingWidget(),
                            ),
                          if (model.isEmpty)
                            SliverToBoxAdapter(
                              child: RequestErrorWidget(
                                null,
                                msg: "暂无评论~",
                                assets: RequestErrorWidget.ASSETS_NO_COMMENT,
                              ),
                            ),
                          if (model.isIdle)
                            SliverToBoxAdapter(
                              child: Row(
                                key: _key,
                                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                children: <Widget>[
                                  Padding(
                                    padding: const EdgeInsets.symmetric(horizontal: 12.0, vertical: 12.0),
                                    child: Text(
                                      "全部评论",
                                      style: Theme.of(context).textTheme.headline3,
                                    ),
                                  ),
                                  PopupMenuButton(
                                    onSelected: (v) {
                                      setState(() {
                                        _sortType = v;
                                        model.sortBy(_sortType);
                                      });
                                    },
                                    itemBuilder: (BuildContext context) {
                                      return _sort
                                          .map((e) => PopupMenuItem(
                                                child: Text(e),
                                                value: e,
                                              ))
                                          .toList();
                                    },
                                    child: Row(
                                      mainAxisSize: MainAxisSize.min,
                                      children: <Widget>[
                                        Text(
                                          _sortType,
                                          style: Theme.of(context).textTheme.subtitle1,
                                        ),
                                        Padding(
                                          padding: const EdgeInsets.fromLTRB(6, 6, 12.0, 6),
                                          child: arrowBottom(),
                                        )
                                      ],
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          if (model.isIdle)
                            SliverList(
                              delegate: SliverChildBuilderDelegate(
                                (context, index) {
                                  return PostCommentWidget(model.list[index]);
                                },
                                childCount: model.list.length,
                              ),
                            ),
                        ],
                      ),
                    ),
                    if (_inputType == 0)
                      Container(
                        decoration: shadowTop(),
                        child: SafeArea(
                          child: Container(
                            height: 50,
                            child: Row(
                              children: <Widget>[
                                Expanded(
                                  child: Row(
                                    children: <Widget>[
                                      Space(
                                        width: 16,
                                      ),
                                      Row(
                                        children: <Widget>[
                                          Image.asset("lib/assets/img/bbs_icon_share.png"),
                                          _padding,
                                          Text(
                                            "转发",
                                            style: TextStyle(color: Color(0xff333333)),
                                          ),
                                        ],
                                      ),
                                      Space(
                                        width: 36,
                                      ),
                                      GestureDetector(
                                        behavior: HitTestBehavior.opaque,
                                        onTap: () async {
                                          Post post = _post;
                                          await (post.isLiked ?? false
                                              ? model.api.postForumUnLike(post.id, "subject_id")
                                              : model.api.postForumLike(post.id, "subject_id"));
                                          setState(() {
                                            _updatePost(2, 0);
                                          });
                                        },
                                        child: Center(
                                          child: Row(
                                            children: <Widget>[
                                              Image.asset("lib/assets/img/bbs_icon_like${_post.isLiked ?? false ? "_complete" : ""}.png"),
                                              _padding,
                                              Text(
                                                "${_post.likeCount}",
                                                style: TextStyle(color: Color(0xff333333)),
                                              ),
                                            ],
                                          ),
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                                Padding(
                                  padding: const EdgeInsets.symmetric(horizontal: 12.0),
                                  child: PrimaryButton(
                                    width: 149,
                                    height: 35,
                                    callback: () {
                                      readyInput();
                                    },
                                    content: "发表评论",
                                  ),
                                )
                              ],
                            ),
                          ),
                        ),
                      ),
                    if (_inputType == 1)
                      input.TextInput(
                        widget.post.id,
                        focusNode: _focusNode,
                        toCommentId: _toComment == null ? null : _toComment.id,
                        user: _toUser,
                        callback: () {
                          resetInput();

                          // if (_sortType == _sort[0]) {
                          //   //最新评论
                          //   model.isEmpty ? model.refresh() : model.loadMore();
                          // } else if (model.list.isEmpty) {
                          //   model.initData();
                          // }
                          model.initData();
                          // model.isEmpty ? model.refresh() : model.loadMore();
                        },
                      )
                  ],
                ));
          }),
    );
  }

  Widget _buildPostDetailWidget(Post post) {
    double width = MediaQuery.of(context).size.width - 24;
    return Container(
      padding: EdgeInsets.fromLTRB(12.0, 12.0, 12.0, 6),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          GestureDetector(
            onTap: () => NavigatorUtil.goSocialUserDetail(context, PostUser(id: post.userId, name: post.nickname, avatar: post.avatar)),
            child: Row(
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                CircleAvatar(backgroundImage: userAvatarProvider(post.avatar), radius: 18),
                Space(
                  width: 8,
                ),
                Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Text(
                      post.nickname,
                      style: Theme.of(context).textTheme.subtitle1.copyWith(fontWeight: FontWeight.w600),
                    ),
                    Text(DateFormat.formatTime(post.createTime), style: Theme.of(context).textTheme.bodyText1)
                  ],
                )
              ],
            ),
          ),
          GestureDetector(
            onLongPress: () async {
              await showActionDialog(context, {
                "复制文字": () {
                  Clipboard.setData(ClipboardData(text: post.content));
                  ToastUtil.show("已复制到粘贴板");
                }
              });
            },
            child: Padding(
              padding: const EdgeInsets.symmetric(vertical: 12.0),
              child: Text(post.content, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16)),
            ),
          ),
          if (post.images.length > 0)
            GridView.count(
              physics: new NeverScrollableScrollPhysics(),
              shrinkWrap: true,
              crossAxisSpacing: 10.0,
              mainAxisSpacing: 10.0,
              childAspectRatio: post.images.length == 1 ? max(16 / 10, post.images[0].getImageAspectRatio()) : 1,
              crossAxisCount: min(3, post.images.length),
              children: post.images
                  .asMap()
                  .keys
                  .map((imageIndex) => GestureDetector(
                        onTap: () => open(context, imageIndex, post.images),
                        child: Hero(
                            tag: post.images[imageIndex].id,
                            child: post.images.length == 1
                                ? Row(
                                    mainAxisSize: MainAxisSize.min,
                                    children: <Widget>[
                                      ClipRRect(
                                          borderRadius: BorderRadius.circular(6),
                                          child: Stack(
                                            children: <Widget>[
                                              CachedNetworkImage(
                                                alignment: Alignment.centerLeft,
                                                imageUrl: post.images[imageIndex].thumbnail,
                                                fit: BoxFit.cover,
                                                width: post.images[imageIndex].getWidth(width),
                                              ),
                                              if (post.images[imageIndex].isLongImage())
                                                Positioned(
                                                  bottom: 4,
                                                  right: 4,
                                                  child: Container(
                                                    padding: EdgeInsets.symmetric(horizontal: 8, vertical: 2),
                                                    decoration:
                                                        BoxDecoration(color: Colors.black.withOpacity(.8), borderRadius: BorderRadius.all(Radius.circular(20))),
                                                    child: Text(
                                                      "长图",
                                                      style: Theme.of(context).textTheme.bodyText1.copyWith(color: Colors.white),
                                                    ),
                                                  ),
                                                )
                                            ],
                                          ))
                                    ],
                                  )
                                : ClipRRect(
                                    borderRadius: BorderRadius.circular(6),
                                    child: CachedNetworkImage(
                                      imageUrl: post.images[imageIndex].thumbnail,
                                      fit: BoxFit.cover,
                                    ),
                                  )),
                      ))
                  .toList(),
            ),
          Space(
            height: 10,
          ),
          Divider()
        ],
      ),
    );
  }
}
