import 'dart:ui';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:sport/bean/forum.dart';
import 'package:sport/bean/post.dart';
import 'package:sport/pages/social/post_widget.dart';
import 'package:sport/provider/lib/provider_widget.dart';
import 'package:sport/provider/lib/view_state_lifecycle.dart';
import 'package:sport/provider/social_detail_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/router/routes.dart';
import 'package:sport/services/userid.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/error.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/persistent_header.dart';
import 'package:sport/widgets/space.dart';

class SocialDetailPage extends StatefulWidget {
  final Forum forum;

  SocialDetailPage(this.forum);

  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends ViewStateLifecycle<SocialDetailPage, SocialDetailModel> with TickerProviderStateMixin, UserId {
  ScrollController _controller;
  double _expandedHeight = 0;
  int _brightness = 0;

  TabController _tabController;
  final List<String> _tabs = ['热门', '最新', '精华', '我的'];

  @override
  SocialDetailModel createModel() => SocialDetailModel(widget.forum.forumId);

  @override
  void initState() {
    super.initState();

    _tabController = TabController(length: _tabs.length, vsync: this)
      ..addListener(() {
        if (_tabController.index.toDouble() == _tabController.animation.value) {
          model?.swtichTab(_tabController.index);
          _controller.animateTo(0, duration: Duration(milliseconds: 100), curve: Curves.ease);
        }
      });
    _controller = ScrollController()
      ..addListener(() {
        if (_controller.position.pixels >= _expandedHeight - 70) {
          if (_brightness == 0) {
            setState(() {
              _brightness = 1;
            });
          }
        } else {
          if (_brightness == 1) {
            setState(() {
              _brightness = 0;
            });
          }
        }
      });
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
    _tabController?.dispose();
    PaintingBinding.instance.imageCache.clear();
  }

  @override
  Widget build(BuildContext context) {
    _expandedHeight = MediaQuery.of(context).size.width * 154.0 / 375;
    return Scaffold(
        backgroundColor: Colors.white,
        body: ProviderWidget<SocialDetailModel>(
          model: model,
          onModelReady: (model) => model.initData(),
          builder: (_, model, __) {
            return EasyRefresh.custom(
              controller: model.refreshController,
              enableControlFinishRefresh: true,
              enableControlFinishLoad: true,
              scrollController: _controller,
              onRefresh: () => model.refresh(),
              onLoad: model.isIdle ? () => model.loadMore() : null,
              header: buildClassicalHeader(
                extent: 100.0 + MediaQuery.of(context).padding.top,
                triggerDistance: 120.0 + MediaQuery.of(context).padding.top,
              ),
              footer: buildClassicalFooter(),
              slivers: <Widget>[
                SliverAppBar(
                  expandedHeight: _expandedHeight,
                  pinned: true,
                  elevation: 0,
                  iconTheme: IconThemeData(color: _brightness == 0 ? Colors.white : Colors.black),
                  brightness: _brightness == 0 ? Brightness.dark : Brightness.light,
                  title: _brightness == 0 ? Text(""):Text("${widget.forum.gameName}", style: titleStyle, ),
                  backgroundColor: Colors.white,
                  flexibleSpace: FlexibleSpaceBar(
                    collapseMode: CollapseMode.pin,
                    background: _buildHeaderWidget(),
                  ),
                  leading: IconButton(
                    icon: Image.asset("lib/assets/img/topbar_return${_brightness == 0?"_white":""}.png"),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  ),
                  actions: <Widget>[
                    IconButton(
                      icon: Text(
                        "发帖",
                        style: TextStyle(fontSize: 15, color: Theme.of(context).accentColor),
                      ),
                      onPressed: () async {
                        var result = await NavigatorUtil.go(context, "${Routes.socialPost}?id=${widget.forum.forumId}");
                        if (result != null) {
                          if (_tabController.index != 1) {
                            _tabController.animateTo(1);
                          } else {
                            model?.swtichTab(_tabController.index);
                            _controller.animateTo(0, duration: Duration(milliseconds: 100), curve: Curves.ease);
                          }
                        }
                      },
                    )
                  ],
                ),
                SliverToBoxAdapter(
                  child: _buildHeaderListWidget(),
                ),
                SliverPersistentHeader(
                  delegate: PersistentHeader(
                      min: 40,
                      max: 40,
                      child: Container(
                        color: Colors.white,
                        padding: EdgeInsets.only(bottom: 5),
                        child: TabBar(
                          isScrollable: true,
                          indicatorPadding: EdgeInsets.symmetric(horizontal: 6),
                          indicatorWeight: 3,
                          controller: _tabController,
                          tabs: _tabs.map((e) => Tab(text: e)).toList(),
                        ),
                      )),
                  pinned: true,
                ),
                if (model.isBusy)
                  SliverToBoxAdapter(
                    child: RequestLoadingWidget(),
                  ),
                if (model.isEmpty)
                  SliverToBoxAdapter(
                    child: RequestErrorWidget(
                      null,
                      msg: "暂无帖子~",
                      assets: RequestErrorWidget.ASSETS_NO_MOTION,
                    ),
                  ),
                if (model.isIdle)
                  SliverList(
                    delegate: SliverChildBuilderDelegate(
                      (context, index) {
                        Post post = model.list[index];
                        return PostWidget(post, model, selfId == post.userId);
                      },
                      childCount: model.list.length,
                    ),
                  ),
              ],
            );
          },
        ));
  }

  Widget _buildHeaderWidget() {
    return Stack(children: <Widget>[
      CachedNetworkImage(
        imageUrl: "${widget.forum.cover}",
        fit: BoxFit.cover,
        width: double.infinity,
        height: MediaQuery.of(context).size.width * 154.0 / 375 + MediaQuery.of(context).padding.top,
      ),
      BackdropFilter(
        filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
        child: Center(
          child: Container(
            color: Colors.red.withOpacity(0),
          ),
        ),
      ),
      Align(
        alignment: Alignment.center,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.fromLTRB(12.0, 50, 12.0, 0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  CircleAvatar(backgroundImage: CachedNetworkImageProvider("${widget.forum.cover}"), radius: 30.0),
                  Space(
                    width: 12,
                  ),
                  Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text(
                        "${widget.forum.gameName}",
                        style: Theme.of(context).textTheme.headline4,
                      ),
                      Space(
                        height: 10,
                      ),
                      Row(
                        children: <Widget>[
                          Image.asset("lib/assets/img/bbs_icon_reportnumber.png"),
                          Space(
                            width: 4,
                          ),
                          Text(
                            "${widget.forum.subjectCount}",
                            style: Theme.of(context).textTheme.subtitle2.copyWith(color: Colors.white),
                          ),
                        ],
                      ),
                    ],
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
      Positioned(
        left: 0,
        right: 0,
        bottom: -1,
        child: Container(
          height: 10,
          decoration: BoxDecoration(borderRadius: BorderRadius.vertical(top: Radius.circular(10)), color: Colors.white),
        ),
      ),
    ]);
  }

  Widget _buildHeaderListWidget() {
    return Container(
      decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.vertical(top: Radius.circular(10))),
      child: Column(
        children: <Widget>[
          Space(
            height: 15,
          ),
          Row(
            children: <Widget>[
              Container(
                  decoration: BoxDecoration(
                      color: Colors.white,
                      borderRadius: BorderRadius.all(Radius.circular(2)),
                      border: Border.all(
                        color: Theme.of(context).accentColor,
                        width: 1,
                      )),
                  padding: EdgeInsets.symmetric(horizontal: 2),
                  margin: EdgeInsets.fromLTRB(12.0, 5, 12, 5),
                  child: Text(
                    "榜单",
                    style: Theme.of(context).textTheme.subtitle2.copyWith(color: Theme.of(context).accentColor),
                  )),
              Text("本月战绩评分榜", style: Theme.of(context).textTheme.subtitle1)
            ],
          ),
          Row(
            children: <Widget>[
              Container(
                  decoration: BoxDecoration(
                      color: Colors.white,
                      borderRadius: BorderRadius.all(Radius.circular(2)),
                      border: Border.all(
                        color: Color(0xff5498FF),
                        width: 1,
                      )),
                  padding: EdgeInsets.symmetric(horizontal: 2),
                  margin: EdgeInsets.fromLTRB(12.0, 5, 12, 5),
                  child: Text(
                    "置顶",
                    style: Theme.of(context).textTheme.subtitle2.copyWith(color: Color(0xff5498FF)),
                  )),
              Text("6月最新版本更新分析", style: Theme.of(context).textTheme.subtitle1)
            ],
          ),
          Row(
            children: <Widget>[
              Container(
                  decoration: BoxDecoration(
                      color: Colors.white,
                      borderRadius: BorderRadius.all(Radius.circular(2)),
                      border: Border.all(
                        color: Color(0xff00DC42),
                        width: 1,
                      )),
                  padding: EdgeInsets.symmetric(horizontal: 2),
                  margin: EdgeInsets.fromLTRB(12.0, 5, 12, 5),
                  child: Text(
                    "公告",
                    style: Theme.of(context).textTheme.subtitle2.copyWith(color: Color(0xff00DC42)),
                  )),
              Text(
                "版本更新通知",
                style: Theme.of(context).textTheme.subtitle1,
              )
            ],
          ),
          Space(
            height: 10,
          ),
          Divider()
        ],
      ),
    );
  }
}
