import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:sport/bean/comment.dart';
import 'package:sport/bean/post_user.dart';
import 'package:sport/provider/comment_sub_list_model.dart';
import 'package:sport/provider/lib/provider_widget.dart';
import 'package:sport/provider/lib/view_state_lifecycle.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/utils/DateFormat.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/dialog/modal_bottom_action.dart';
import 'package:sport/widgets/error.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';
import 'package:sport/widgets/text_input.dart' as input;

class CommentDialog extends StatefulWidget {
  final String postId;
  final Comment comment;
  final bool showInput;

  CommentDialog(this.postId, this.comment, {this.showInput = false});

  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends ViewStateLifecycle<CommentDialog, CommentSubListModel> {
  var _padding = Space(
    width: 5,
  );

  FocusNode _focusNode;
  Comment _toComment;
  PostUser _toUser;

  @override
  void initState() {
    _focusNode = FocusNode();
    super.initState();
  }

  @override
  void dispose() {
    _focusNode?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        Container(
          width: double.infinity,
          height: 50,
          child: Stack(
            alignment: Alignment.center,
            children: <Widget>[
              Text(
                "回复详情",
                style: TextStyle(fontSize: 16),
              ),
              Positioned(
                  left: 0,
                  child: IconButton(
                    icon: arrowBack(),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  ))
            ],
          ),
        ),
        Expanded(child: _buildCommentListWidget(model, widget.comment)),
        input.TextInput(
          widget.postId,
          focusNode: _focusNode,
          parentCommentId: widget.comment.id,
          comment: widget.showInput,
          toCommentId: _toComment == null ? null : _toComment.id,
          user: _toUser,
          callback: () {
            _toComment = null;
            _toUser = null;
            model.initData();
            widget.comment.commentCount += 1;
          },
        )
      ],
    );
  }

  @override
  CommentSubListModel createModel() => CommentSubListModel(widget.comment);

  Widget _buildCommentListWidget(CommentSubListModel subModel, Comment parent) {
    return ProviderWidget<CommentSubListModel>(
      model: subModel,
      onModelReady: (model) => model.initData(),
      builder: (_, model, __) => EasyRefresh.custom(
          controller: model.refreshController,
          enableControlFinishRefresh: true,
          enableControlFinishLoad: true,
          onLoad: model.isIdle ? () => model.loadMore() : null,
          footer: buildClassicalFooter(),
          slivers: <Widget>[
            SliverToBoxAdapter(
              child: _buildCommentListHeaderWidget(parent),
            ),
            if (model.isBusy)
              SliverToBoxAdapter(
                child: RequestLoadingWidget(),
              ),
            if (model.isEmpty)
              SliverToBoxAdapter(
                child: RequestErrorWidget(
                  null,
                  msg: "暂无评论~",
                  assets: RequestErrorWidget.ASSETS_NO_COMMENT,
                ),
              ),
            if (model.isIdle)
              SliverList(
                delegate: SliverChildBuilderDelegate(
                  (context, index) {
                    if (index.isOdd)
                      return Divider(
                        height: 1,
                        indent: 32,
                      );
                    return _buildCommentListItemWidget(model.list[index ~/ 2]);
                  },
                  childCount: model.list.length * 2 - 1,
                ),
              ),
            SliverToBoxAdapter(
              child: SizedBox(height: 30,),
            ),
          ]),
    );
  }

  Widget _buildCommentWidget(Comment item) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        item.socialInfo == null
            ? Container()
            : Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  GestureDetector(
                      behavior: HitTestBehavior.opaque,
                      onTap: () => NavigatorUtil.goSocialUserDetail(context, item.socialInfo),
                      child: Row(
                        children: <Widget>[
                          CircleAvatar(backgroundImage: userAvatarProvider(item.socialInfo?.avatar), radius: 18),
                          Space(
                            width: 8,
                          ),
                          Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              Text(
                                item.socialInfo?.name,
                                style: Theme.of(context).textTheme.subtitle1.copyWith(fontWeight: FontWeight.bold),
                              ),
                              SizedBox(height: 4,),
                              Text(DateFormat.formatTime(item.createTime), style: Theme.of(context).textTheme.bodyText1)
                            ],
                          ),
                        ],
                      )),
                  GestureDetector(
                    behavior: HitTestBehavior.opaque,
                    onTap: () async {
                      await model.api.postForumLike(item.id, "comment_id");
                      setState(() {
                        item.toggleLike();
                      });
                    },
                    child: Row(
                      children: <Widget>[
                        Image.asset("lib/assets/img/bbslist_icon_like${item.isLiked ?? false ? 'd' : ''}.png"),
                        _padding,
                        Text(item.likeCount == 0 ? "" : "${item.likeCount}"),
                      ],
                    ),
                  ),
                ],
              ),
        GestureDetector(
          onTap: () {
            setState(() {
              _toComment = item;
              _toUser = item.socialInfo;
              _focusNode?.requestFocus();
            });
          },
          onLongPress: () async {
            await showActionDialog(context, {
              "复制文字": () {
                Clipboard.setData(ClipboardData(text: item.content));
                ToastUtil.show("已复制到粘贴板");
              }
            });
          },
          child: Padding(
              padding: const EdgeInsets.symmetric(vertical: 12.0),
              child: RichText(
                text: TextSpan(style: DefaultTextStyle.of(context).style, children: <InlineSpan>[
                  if (item.toUserName?.isNotEmpty == true)
                    TextSpan(
                      text: '@${item.toUserName} ',
                      style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16, color: Theme.of(context).accentColor),
                      // recognizer: TapGestureRecognizer()..onTap = () {
                      //   print("111111111111");
                      //   setState(() {
                      //     _toUser = PostUser(id: item.toUserId, name: item.toUserName);
                      //   });
                      // }
                    ),
                  TextSpan(text: item.content, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16)),
                ]),
              )),
        ),
      ],
    );
  }

  Widget _buildCommentListHeaderWidget(Comment item) {
    return Column(
      children: <Widget>[
        Padding(
          padding: EdgeInsets.fromLTRB(12.0, 6.0, 12.0, 6.0),
          child: _buildCommentWidget(item),
        ),
        Divider(
          height: 1,
        )
      ],
    );
  }

  Widget _buildCommentListItemWidget(Comment item) {
    return Padding(
      padding: EdgeInsets.fromLTRB(32.0, 12.0, 12.0, 0.0),
      child: _buildCommentWidget(item),
    );
  }
}

Future showCommentList(BuildContext context, Comment currentItem, {bool comment = false}) async {
  return await showModalBottomSheet(
    context: context,
    isScrollControlled: true,
    backgroundColor: Colors.white,
    shape: RoundedRectangleBorder(borderRadius: BorderRadius.vertical(top: Radius.circular(10))),
    builder: (BuildContext context) {
      return SingleChildScrollView(
        child: Container(
          height: max(520, MediaQuery.of(context).size.height / 3 * 2),
          padding: EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom), // !important
          child: CommentDialog(currentItem.subjectId, currentItem, showInput: comment),
        ),
      );
    },
  );
}
